@@ -54,31 +54,14 @@ class BrowserIntegrationTest(TestCase):
         driver.quit()
 
     def test_chrome_can_access_requests(self):
-        proxy = '167.172.180.46:35301'
-        url = 'https://datadome.co/customers-stories/toppreise-ends-web-' \
-              'scraping-and-content-theft-with-datadome/'
-        sw_options = {
-            'proxy': {
-                'https': f'https://{proxy}'
-            }
-        }
-        from seleniumwire.webdriver import ChromeOptions
-        chrome_options = ChromeOptions()
-        driver = webdriver.Chrome(  # Optimized for bot detection
-            options=chrome_options,
-            seleniumwire_options=sw_options
-        )
+        url = 'https://www.wikipedia.org/'
+        driver = webdriver.Chrome()
         driver.get(url)
 
-        import time
-        time.sleep(30)
-        # request = driver.wait_for_request(url)
-        #
-        # self.assertEqual(request.response.status_code, 200)
-        # self.assertIn('text/html', request.response.headers['Content-Type'])
-
-        print(driver.requests)
+        request = driver.wait_for_request(url)
 
+        self.assertEqual(request.response.status_code, 200)
+        self.assertIn('text/html', request.response.headers['Content-Type'])
 
         driver.quit()
 
